@@ -1,6 +1,6 @@
 package Net::WebSocket;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 =encoding utf-8
 
@@ -10,19 +10,23 @@ Net::WebSocket - WebSocket in Perl
 
 =head1 SYNOPSIS
 
+    use Net::WebSocket::Handshake::Client ();
+    use Net::WebSocket::HTTP_R ();
+
     my $handshake = Net::WebSocket::Handshake::Client->new(
         uri => $uri,
     );
 
-    syswrite $inet, $handshake->create_header_text() . "\x0d\x0a" or die $!;
+    syswrite $inet, $handshake->to_string() or die $!;
 
     #You can parse HTTP headers however you want;
     #Net::WebSocket makes no assumptions about this.
-    my $req = HTTP::Response->parse($hdrs_txt);
+    my $resp = HTTP::Response->parse($hdrs_txt);
 
-    #XXX More is required for the handshake validation in production!
-    my $accept = $req->header('Sec-WebSocket-Accept');
-    $handshake->validate_accept_or_die($accept);
+    #If you use an interface that’s compatible with HTTP::Response,
+    #then you can take advantage of this convenience function;
+    #otherwise you’ll need to do a bit more work.
+    Net::WebSocket::HTTP_R::handshake_parse_response( $handshake, $resp );
 
     #See below about IO::Framed
     my $parser = Net::WebSocket::Parser->new(
@@ -67,20 +71,19 @@ a UNIX socket, ordinary TCP/IP, some funky C<tie()>d object, or whatever.
 Net::WebSocket also “has no opinions” about how you should do I/O or HTTP
 headers. There are too many different ways to accomplish HTTP header
 management in particular for it to be sensible for a WebSocket library to
-impose any one approach. As a result of this, Net::WebSocket can likely
-fit your project; however, it won’t absolve you of the need to know some
-things about the WebSocket protocol itself. There are some examples
+impose any one approach. As a result of this, Net::WebSocket can probably
+fit your project with minimal overhead. There are some examples
 of how you might write complete applications (client or server)
-in the distribution’s C<demo/> directory.
+in the distribution’s F<demo/> directory.
 
-Net::WebSocket is not a “quick” WebSocket solution; for that,
-check out L<Mojolicious>. Net::WebSocket’s purpose is to support anything
+Net::WebSocket emphasizes flexibility and lightness rather than the more
+monolithic approach in modules like L<Mojolicious>.
+Net::WebSocket should support anything
 that the WebSocket protocol itself can do, as lightly as possible and without
 prejudice as to how you want to do it: extensions, blocking/non-blocking I/O,
 arbitrary HTTP headers, etc. Net::WebSocket will likely require more of an
-investment up-front, but its flexibility should allow it to do anything that
-can be done with WebSocket, and much more cleanly than a more “monolithic”
-solution would likely allow.
+investment up-front, but the end result should be a clean, light
+implementation that will grow (or shrink!) as your needs dictate.
 
 =head1 OVERVIEW
 
@@ -92,24 +95,25 @@ Here are the main modules:
 
 =item L<Net::WebSocket::Handshake::Client>
 
-Logic for handshakes. These are probably most useful in tandem with
-modules like L<HTTP::Request> and L<HTTP::Response>.
+Logic for handshakes. Every application needs one of these. As of version
+0.5 this handles all headers and can also do
+subprotocol and extension negotiation for you.
 
+=item L<Net::WebSocket::HTTP_R>
+
+A thin convenience wrapper for L<HTTP::Request> and L<HTTP::Response>,
+CPAN’s “standard” classes to represent HTTP requests and responses.
+Net::WebSocket::HTTP_R should also work with other classes whose
+interfaces are compatible with these “standard” ones.
 
 =item L<Net::WebSocket::Endpoint::Server>
 
 =item L<Net::WebSocket::Endpoint::Client>
 
-The highest-level abstraction that this distribution provides. It parses input
-and responds to control frames and timeouts. You can use this to receive
-streamed (i.e., fragmented) transmissions as well.
-
-=item L<Net::WebSocket::Streamer::Server>
-
-=item L<Net::WebSocket::Streamer::Client>
-
-Useful for sending streamed (fragmented) data rather than
-a full message in a single frame.
+A high-level abstraction to parse input
+and respond to control frames and timeouts. You can use this to receive
+streamed (i.e., fragmented) transmissions as well. You don’t have to use
+this module, but it will make your life easier.
 
 =item L<Net::WebSocket::Parser>
 
@@ -122,6 +126,13 @@ Useful for creating raw frames. For data frames (besides continuation),
 these will be your bread-and-butter. See L<Net::WebSocket::Frame::text>
 for sample usage.
 
+=item L<Net::WebSocket::Streamer::Server>
+
+=item L<Net::WebSocket::Streamer::Client>
+
+Useful for sending streamed (fragmented) data rather than
+a full message in a single frame.
+
 =back
 
 =head1 IMPLEMENTATION NOTES
@@ -130,11 +141,13 @@ for sample usage.
 
 WebSocket uses regular HTTP headers for its handshakes. Because there are
 many different solutions around for parsing HTTP headers, Net::WebSocket
-tries to be “agnostic” about how that’s done. The liability of this is
-that you, the library user, will need to implement some of the handshake
-logic yourself. If you’re building from the ground up that’s not a lot of
-fun, but if you’ve already got a solution in place for parsing headers then
-Net::WebSocket can fit into that quite easily.
+is “agnostic” about how that’s done. The advantage is that if you’ve got
+a custom solution for parsing headers then Net::WebSocket can fit into
+that quite easily.
+
+The liability of this is that you, the library user, must give headers
+directly to your Handshake object. (NB: L<Net::WebSocket::HTTP_R> might
+be able to do this for you.)
 
 =head2 Masking
 
@@ -196,14 +209,20 @@ can do in your application.
 
 =back
 
+=head2 permessage-deflate
+
+Net::WebSocket 0.5 introduces support for the permessage-delate extension
+to allow compressed messages over WebSocket. See
+L<Net::WebSocket::PMCE::deflate> for more details.
+
 =head1 TODO
 
+At this point Net::WebSocket should support every widely implemented
+WebSocket feature.
+
 =over
 
-=item * Support the L<permessage-deflate|https://tools.ietf.org/html/rfc7692>
-extension natively.
-
-=item * Add tests, especially for extension support.
+=item * Add more tests.
 
 =back
 
